Package: lambda.r
Type: Package
Title: Modeling Data with Functional Programming
Version: 1.2.4
Date: 2019-09-15
Depends: R (>= 3.0.0)
Imports: formatR
Suggests: testit
Author: Brian Lee Yung Rowe
Maintainer: Brian Lee Yung Rowe <r@zatonovo.com>
Description: A language extension to efficiently write functional programs in R. Syntax extensions include multi-part function definitions, pattern matching, guard statements, built-in (optional) type safety.
License: LGPL-3
LazyLoad: yes
NeedsCompilation: no
Packaged: 2019-09-15 08:08:15 UTC; brian
Repository: CRAN
Date/Publication: 2019-09-18 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 12:41:53 UTC; unix
