importFrom("utils", "capture.output", "getParseData", "str", "tail")
importFrom("formatR", "tidy_source")
export("%as%")
export("%:=%")
export("%::%")
export("UseFunction")
export("NewObject")
export("describe")
export("EMPTY")
S3method(print, lambdar.fun)
S3method(print, lambdar.type)
export("debug.lr")
export("undebug.lr")
export("undebug.all")
export("is.debug")
export("which.debug")
export("seal")
export("%isa%")
export("%hasa%")
export("%hasall%")

